# -*- coding: UTF-8 -*-


#!! 歯面作図スクリプト
#      icd_gear.py	2022.8.12  kita

# 必要なファイル
#   my_module.py  my_vector.py  icd_module.py  (icd_gear.icd)


# 履歴
# 2022.8.20     アルゴリズム見直し円弧走査線方式

import tkinter as tk
from tkinter import ttk
import os,sys
import my_module as mmd
import my_vector as myv
import icd_module as imd
import math
import numpy as np
from numpy import linalg as LA
import zen_han      #全角<-->半角変換

import clr
clr.AddReference('sxnet')
from sxnet import *

pi = np.pi
# 保存ファイル名取得
tmp_fn = os.path.basename(__file__)
save_fn = tmp_fn.replace('.py','.dat')

entry_txt = ['16', '0.0', '30', '0.1']  # 歯数, 転移係数,分割数、バックラッシュ


def disp_ent():
    (mj, nz, x_t, nb, bcl, debug) = get_ent()
    print(' Module= %5.3f   歯数= %d  転移係数= %5.3f  分割数= %d  BackLash= %3.1f Debug= %d' % (mj, nz, x_t, nb, bcl, debug))


def push_exe():
    global debug
    (mj, nz, x_t, nb, bcl, debug) = get_ent()
    disp_ent()
    alf = 20		# 圧力角[deg]
    pcd = mj * nz       # 基準円直径
    alf_rad = np.deg2rad(alf)	
    inv = np.tan(alf_rad)-alf_rad;
    kf  = alf_rad/pi;
    # dn  = nint(nz*kf + 1.5); */			/* 作図歯数 */
    dn  = 4        # 作図歯数 2では足りない 
    nn = nb * dn    # 分割数 30

    pich = pi*mj	# ピッチ 
    rp = mj*nz/2	# 基準ピッチ円半径 
    rs = rp + mj*(1 + x_t)  # 歯先半径 
    rm = rp - mj*(1.25-x_t)   # 歯元半径 */
    th = dn*360/nz
    dth = th/nn
    dlen = np.array([dn*pich/nn,0.0])   # 微小水平移動量

    #bcl_y = bcl/4/np.sin(alf_rad) # バックラッシュ用デルタy
    bcl_x = bcl/4/np.cos(alf_rad) # バックラッシュ用デルタx

    # 直線の初期値
    a = np.tan(alf_rad-pi/2)   # 傾き
    b = rp + mj*x_t - a*pich/4  # 切辺
    y0 = rs
    x0 = (y0-b)/a
    # ラック刃先エッジ
    p00 = np.array([x0-bcl_x, y0])

    # ラック歯元円弧半径, 歯面直線と歯底円弧の交点、歯底円弧中心座標
    (rrc, p01, p02) = get_arc(mj, rp, x_t)
    p01[0] = p01[0] - bcl_x
    p02[0] = p02[0] - bcl_x
    #print('mj= %2.3f rp= %3.3f x_t= %1.1f  rrc= %2.3f p01= %3.3f, %3.3f  p02= %3.3f, %3.3f' % \
    #        (mj, rp, x_t, rrc, p01[0], p01[1], p02[0], p02[1]))
    
    p03 = np.array([mj*pi/2-bcl_x, rm])   #歯底真下
    #return

    # ラック直線部、歯元円弧の軌跡リスト
    prc_list = []   # p0:歯面直線(歯先側) p1: 歯面直線(歯元側)　p2: 歯元円弧中心
    ang01 = 180.0+alf   # p01角度[deg]
    ang02 = 270.0       # p02角度[deg]
    angle = 90.0 - alf  # 半円弧角度[deg]

    for i in range(nn):
        ll = (i-nn/2)*dlen
        tmp_p0 = p00+ll
        tmp_p1 = p01+ll
        tmp_p2 = p02+ll
        tmp_p3 = p03+ll

        th = (i-nn/2)*dth 
        th_rad = np.deg2rad(th)
        tmp_p0 = myv.rotate2(tmp_p0,th_rad)
        tmp_p1 = myv.rotate2(tmp_p1,th_rad)
        tmp_p2 = myv.rotate2(tmp_p2,th_rad)
        tmp_p3 = myv.rotate2(tmp_p3,th_rad)
        
        prc_list.append([tmp_p0, tmp_p1, tmp_p2])
        # Debug
        if debug == 1:
            if i-nn/2 < 0:
                imd.p2p_line(tmp_p0, tmp_p1-tmp_p0, '緑')
            else:
                imd.p2p_line(tmp_p0, tmp_p1-tmp_p0, '赤')

            center = SxPos(tmp_p2[0], tmp_p2[1])
            #s_ang = ang01 + th
            s_pos = SxPos(tmp_p1[0], tmp_p1[1])
            seg = SxEntSeg.createArc2D(center,s_pos,angle)

    #return

    # 円弧走査作業
    bai = 50    # 歯元を細かく分割する
    nnh = nb * bai     #走査円弧分割数
    #nnh = 20     #
    dr = 2.25 * mj / nnh    
    #dr1 = dr/10   # 歯元は細かく分割

    phm_list = []   # 歯面交点リスト
    poshm_list = [] # 歯面交点リスト SxPos
    phm1 = np.array([rm*np.sin(pi/nz), rm*np.cos(pi/nz)])

    phm_list.append(phm1)
    poshm_list.append(SxPos(phm1[0], phm1[1]))


    for i in range(1, nnh+1):
        d, m = divmod(i, bai)   
        if d != 0 and m != 0:
            continue

        rscn = rm + dr*i
        #print(i, rscn) #!!!
        tmp_list = []
        cnt = 0
        for p02 in prc_list:
            xp = get_xp(rscn, rrc, p02) # 走査円弧半径、歯元円半径、p0,p1,p2
            
            #print(i, cnt, xp)   #!!!
            #yn = input('Nexe')  #!!!

            if xp[0] != None:
                tmp_list.append(xp)
            cnt += 1

        tmp_slist = sorted(tmp_list, key=lambda x: x[0])       #x座標値sort
        phm_list.append(tmp_slist[0])   # x座標最小の点とリストに追加
        poshm_list.append(SxPos(phm_list[-1][0],phm_list[-1][1]))

        if debug == 1:
            pos = SxPos(tmp_slist[0][0], tmp_slist[0][1])
            SxEntSeg.createPoint2D(pos)

    
    #いよいよ歯面作図作業開始


    SxSys.setLayer(1)   #書き込みレヤーの設定
    # 歯面作図(Spline)
    ang = 180-180/nz
    #seg = SxEntSeg.createSpline2D(poshm_list,SxEntSeg.SPLINE_KIND_FIXED,True,False,ang,0.0)
    seg0 = SxEntSeg.createSpline2D(poshm_list,SxEntSeg.SPLINE_KIND_NORMAL,False,False,0.0,0.0)
    imd.change_seg_pen(seg0, '薄緑', '実線', '中線')    # seg0はリスト
    rpart0 = SxEntRPart.create(seg0, 'gear0', 0) 
    
    # 反転コピー
    pos1 = SxPos(0.0, 0.0)
    pos2 = SxPos(0.0, rp)
    #rpart1 = SxEnt.copy2DMirror(rpart0, pos1, pos2, True, True, True)
    rpart1 = rpart0.copy2DMirror(pos1, pos2, True, True, True)
    #SxEnt.copy2DMirror(seg0, pos1, pos2, True, True, True)  #戻り値なし
    #rpart0.add(rpart1) 

    # 歯先円弧作図
    p0 = phm_list[-1]
    th = np.arccos(np.clip(p0[1]/rs, -1.0, 1.0))
    center = SxPos(0.0, 0.0, 0.0)	# 円心座標オブジェクト
    s_pos  = SxPos(p0[0], p0[1])	# 円弧開始座標オブジェクト
    angle = 2*np.rad2deg(th)		        #円弧相対角度
    seg2 = SxEntSeg.createArc2D(center,s_pos,angle)

    #print(type(seg0), type(seg2))   #!!!
    imd.change_seg_pen([seg2], '薄緑', '実線', '中線')
    
    rpart0.add([rpart1, seg2]) 
    #rpart0.add([seg2])     # OK!

    #ss_mj = cb_mdl.get()
    #name = 'Gear_m%s_z%d' % (ss_mj, nz) 
    
    #print(type(rpart0), type(rpart1), type(seg2))   #!!! <class 'sxnet.SxEntSeg[]'> <class 'NoneType'> <class 'sxnet.SxEntSeg'>
    #print(type(seg0), type(seg1), type(seg2))   #!!! <class 'sxnet.SxEntSeg[]'> <class 'NoneType'> <class 'sxnet.SxEntSeg'>
    #rpart = SxEntRPart.create(seg0, name, 0)
    #rpart = SxEntRPart.create([seg0+seg1+seg2], name, 0)
    #rpart = SxEntRPart.create(seg0+seg1+seg2, name, 0) # TypeError: unsupported operand type(s) for +: 'SxEntSeg[]' and 'NoneType'
    #rpart = SxEntRPart.create([seg0, seg1, seg2], name, 0)
        #TypeError: No method matches given arguments for create: (<class 'list'>, <class 'str'>, <class 'int'>)

    # 補助線作図
    r4 = rp - 3*mj
    p4 = r4 * phm1/LA.norm(phm1)
    seg3 = imd.p2p_line(p4, phm1 - p4, '薄緑')
    p5 = np.array([-phm1[0], phm1[1]])
    p6 = np.array([-p4[0], p4[1]])
    seg4 = imd.p2p_line(p6, p5 - p6, '薄緑')
    
    s_pos = SxPos(p4[0], p4[1])
    angle = 2*np.rad2deg(np.arccos(np.clip(phm1[1]/rm, -1.0, 1.0)))
    seg5 = SxEntSeg.createArc2D(center,s_pos,angle)

    #imd.change_seg_pen([seg3], '橙', '実線', '中線')
    #imd.change_seg_pen([seg4], '橙', '実線', '中線')
    #imd.change_seg_pen([seg5], '橙', '実線', '中線')
    imd.change_seg_pen([seg3, seg4, seg5], '薄緑', '実線', '中線')
    rpart1 = SxEntRPart.create([seg3, seg4, seg5], 'gear1', 0) 
    #rpart1.add([seg4, seg5])

    # PCD作成
    p7 = rp * p4/LA.norm(p4)
    s_pos = SxPos(p7[0], p7[1])
    seg6 = SxEntSeg.createArc2D(center,s_pos,angle)
    imd.change_seg_pen([seg6], '青緑', '一点鎖線', '細線')

    # 諸元文字列書き出し
    pos = SxPos(2*mj, rp)
    opt = SxOptCreateText() # 文字列要素作成オプション
    opt.text[0] = zen_han.Han2Zen(('m%5.3f z%d x_t=%3.1f' % (mj, nz, x_t)), zen_han.EISU)
    opt.text[1] = zen_han.Han2Zen(('PCD=%6.3f BL=%3.2f' % (2*rp, bcl)), zen_han.EISU) 
    seg = SxEntSeg.createText(pos, opt)
    

    print('\n Module= %4.3f  歯数= %d  転移係数= %3.2f PCD= %6.3f BackLash= %4.3f で作図完了しました\n' % (mj, nz, x_t, 2*rp, bcl))

    return




# 操作円弧ラック軌跡図との交点を求める
def get_xp(rscn, rrc, p02): # 走査円弧半径、歯元円半径、p0,p1,p2
    p0 = p02[0]; p1 = p02[1]; p2 = p02[2]
    if LA.norm(p1) >= rscn: #歯元円弧と交差
        A = p2
        B = np.array([rrc, 0])
        C = np.array([0, rscn])
        (n, ra, rb, rc) = myv.case4(A, B, C)
        if n == 0:
            #print('\n   歯底円との交点がありません')    #!!!
            return [None, None]

        if rc[0][0] < rc[1][0]: # x座標の小さい方
            xp = rc[0]
        else:
            xp = rc[1]
        flg = 'arc' #!!!


    elif LA.norm(p1) < rscn:    # 歯面直線と交差
        A = -p1
        B = np.array([0, rscn])
        C = p0 - p1
        (n, r) = myv.case3(A, B, C)
        
        #imd.p2p_line(-A, C ,'紫')    #!!!
        #imd.p2p_line(-r[0], r[2][0] ,'赤')    #!!!
        
        if n == 0:
            #print('\n   歯面直線との交点がありません')  #!!!
            return [None, None]

        #if r[1][0][0] <= r[1][1][0]: # x座標の小さい方
        if r[1][0][1] >= r[1][1][1]: # y座標の大きい方
            xp = r[1][0]
        else:
            xp = r[1][1]
        flg = 'line'    #!!!

    #print(flg, xp)
    return xp   # 走査円弧 rscn とラック軌跡図との交点座標


# ラック元図取得(r=0.38mj)
def get_arc(mj, rp, x_t):
    alf = 20
    alf_rad = np.deg2rad(alf)
    a = np.tan(alf_rad-pi/2)    #傾き
    b = rp + mj*(x_t-pi*a/4)    #切辺
    #tmp0 = mj*pi*a/2 + b -rp + 1.25*mj
    #tmp1 = a*np.cos(alf_rad)+1-np.sin(alf_rad)
    #r = tmp0/tmp1  # 歯元円弧半径
    rm = rp - mj*(1.25-x_t)
    r = 0.38 * mj
    y1 = rm + r*(1-np.sin(alf_rad))
    x1 = (y1 - b) / a 
    p1 = np.array([x1, y1])
    p2 = np.array([x1+r*np.cos(alf_rad), rm+r])
    
    return (r, p1, p2)  # 歯元円弧半径, 歯面直線と歯底円弧の交点、歯底円弧中心座標


# ラック元図取得 旧
def get_arc00(mj, rp, x_t):
    alf = 20
    alf_rad = np.deg2rad(alf)
    a = np.tan(alf_rad-pi/2)    #傾き
    b = rp-mj*(x_t+1)*pi*a/4    #切辺
    tmp0 = mj*pi*a/2 + b -rp + 1.25*mj
    tmp1 = a*np.cos(alf_rad)+1-np.sin(alf_rad)
    r = tmp0/tmp1  # 歯元円弧半径
    x1 = mj*pi/2 - r*np.cos(alf_rad)
    y1 = a*x1 + b
    p1 = np.array([x1, y1])
    p2 = np.array([mj*pi/2, rp-1.25*mj+r])
    
    return (r, p1, p2)  # 歯元円弧半径, 歯面直線と歯底円弧の交点、歯底円弧中心座標



def get_ent():
    ss_mj = cb_mdl.get()
    if ss_mj == 'CP5':
        #mj = 1.5915
        mj = 5/pi
    else:
        mj = float(ss_mj)

    if v_dbg.get():
        debug = 1
        SxSys.setLayer(53)   #書き込みレヤーの設定
    else:
        debug = 0
        SxSys.setLayer(1)   #書き込みレヤーの設定

    nz = float(entry[0].get()) 
    x_t = float(entry[1].get())
    nb = int(entry[2].get())
    bcl = float(entry[3].get())
    
    return (mj, nz, x_t, nb, bcl, debug)    # Module, 歯数, 転移係数、分割数、バックラッシュ、Debug Mode



def save_data(save_fn):    # 設定値保存
    (mj, nz, x_t, nb, bcl, debug) = get_ent()
    ss_mj = cb_mdl.get()
    ss = '%s,%d,%3.2f,%d,%3.2f,%d' % (ss_mj, nz, x_t, nb, bcl, debug)


    fp = open(save_fn,'w')
    fp.write(ss)
    fp.close()

def read_data(save_fn):    # 設定値の読み込み
    fp = open(save_fn, 'r')
    lines = fp.readlines()
    #print(lines)
    fp.close()
    i = 0
    for line in lines:
        #print(i, line)    #
        line = line.rstrip()    # 行末の改行削除
        if line.find('#') > -1 or line == '' or len(line) < 3 :  # #が含まれてるなら
            continue
        tmp = line.split(',')
        ss_mj = tmp[0]
        ss_nz = tmp[1]
        ss_x_t = tmp[2]
        ss_nb = tmp[3]
        ss_bcl = tmp[4]
        debug = int(tmp[5])

    cb_mdl.set(ss_mj)
    entry[0].delete(0,tk.END)
    entry[0].insert(0,ss_nz)
    entry[1].delete(0,tk.END)
    entry[1].insert(0,ss_x_t)
    entry[2].delete(0,tk.END)
    entry[2].insert(0,ss_nb)
    entry[3].delete(0,tk.END)
    entry[3].insert(0,ss_bcl)
    if debug == 0:
        v_dbg.set(False)
    else:
        v_dbg.set(True)

# 終了時の処理(画面サイズ,位置保存)
def on_closing():
    save_data(save_fn)
    print('\n   %s に設定値を保存して終了します\n' % save_fn)

    SxSys.setLayer(1)   #書き込みレヤーの設定

    ss_geom = root.geometry() 
    mmd.save_geometry(save_fn_geom, ss_geom)
    root.destroy()




#************************************
# ボタン等の部品配置
#************************************
nl = 9
root = tk.Tk()
root.attributes("-topmost", True)    # 常に前面に表示
root.configure(bg='RoyalBlue4')
root.title('icd_gear.py')   #タイトル

fn = os.path.basename(__file__)     # icd_gear.py   
(tmp, ext) = os.path.splitext(fn)   # (icd_gear, py)
save_fn_geom = '%s.ini' % tmp
ss_geom = mmd.read_geometry(save_fn_geom)
if ss_geom == None:
    ss_geom = '150x%d+1000+0' % (nl*26)

root.geometry(ss_geom)


btn_bcol = 'grey21'     #ボタン背景色
btn_tcol = 'white'      #ボタン文字色
label_bcol = 'khaki3'   #ラベル背景色
label_tcol = 'black'   #ラベル文字色
e_bcol = 'khaki4'

x00 = 10; y00 = 4
dx = 100;  dy = 30
wdth = 12 
px = 5; py = 5

entry = []

#   Title
label = tk.Label(root, text='歯面作図', bg='orange',fg=label_tcol)    
label.grid(column=0,row=0,padx=px, pady=py)
label0 = tk.Label(root, text='icd_gear.py', bg='orange',fg=label_tcol)    
label0.grid(column=1,row=0,padx=1, pady=py)

#   Module
label = tk.Label(root, text='Modele', bg=label_bcol,fg=label_tcol)    
label.grid(column=0,row=1,padx=px, pady=py)
v_mdl = tk.StringVar()
cb_mdl = ttk.Combobox(root, width=4,textvariable = v_mdl) #, style="BW.TLabel")
#cb_mdl.bind('<<ComboboxSelected>>', cb_sml_selected)
cb_mdl['values']=('1', '1.5', 'CP5', '2', '2.5', '3', '4', '5', '6', '8','10')
cb_mdl.set('2')
cb_mdl.grid(column=1,row=1,padx=px, pady=py)

#   歯数
label = tk.Label(root, text='歯数', bg=label_bcol,fg=label_tcol)    
label.grid(column=0,row=2,padx=px, pady=py)
entry.append(tk.Entry(root, width=6,justify="right",bg=e_bcol)) 
entry[0].insert(0,entry_txt[0])
entry[0].grid(column=1, row=2, padx=px, pady=py)

#  転移係数 
label = tk.Label(root, text='転移係数', bg=label_bcol,fg=label_tcol)    
label.grid(column=0,row=3,padx=px, pady=py)
entry.append(tk.Entry(root, width=6,justify="right",bg=e_bcol)) 
entry[1].insert(0,entry_txt[1])
entry[1].grid(column=1, row=3, padx=px, pady=py)

#  分割数 
label = tk.Label(root, text='分割数', bg=label_bcol,fg=label_tcol)    
label.grid(column=0,row=4,padx=px, pady=py)
entry.append(tk.Entry(root, width=6,justify="right",bg=e_bcol)) 
entry[2].insert(0,entry_txt[2])
entry[2].grid(column=1, row=4, padx=px, pady=py)

#   バックラッシュ
label = tk.Label(root, text='BackLash', bg=label_bcol,fg=label_tcol)    
label.grid(column=0,row=5,padx=px, pady=py)
entry.append(tk.Entry(root, width=6,justify="right",bg=e_bcol)) 
entry[3].insert(0,entry_txt[3])
entry[3].grid(column=1, row=5, padx=px, pady=py)

# Debug Check Button
v_dbg = tk.BooleanVar()   # 論理値型の変数 Instance
v_dbg.set(False)          # 初期値品名記入なし 
cb_dbg = tk.Checkbutton(root,text='Debug',variable=v_dbg)
cb_dbg.grid(column=0, row=6, padx=px, pady=py)



# 実行Button
btn_exe = tk.Button(root,text = '実行', command = push_exe,bg=btn_bcol,fg=btn_tcol) #ボタンの作成
btn_exe.grid(column=1, row=6, padx=px, pady=py)

# 起動時の処理
if os.path.exists(save_fn):      # fnが存在するなら
    try:
        read_data(save_fn)
    except:
        print('%s がないので、初期データをセットします' % save_fn)

#if len(sys.argv) < 2:   #引き数無し ###引数無 ###argv
#    debug = 0
#else:    # 引数１個 
#    debug = 1   # Debug Mode
    #debug = int(sys.argv[1])
#    label0['text'] = 'Debug Mode'


root.protocol("WM_DELETE_WINDOW", on_closing)
root.mainloop()
